#include "atoms.hh"
#include "rom.hh"

void Block::Write(unsigned offs, unsigned color_offs) const
{
    for(unsigned a=0; a<4; ++a)
        ROM[offs + a] = c[a].tile;
        
    unsigned palette =
        (c[0].palette<<0) |
        (c[2].palette<<2) |
        (c[1].palette<<4) |
        (c[3].palette<<6);
        
    ROM[color_offs] = palette;
}

void Block::Load(unsigned offs, unsigned color_offs)
{
    unsigned char palettebyte = ROM[color_offs];
    for(unsigned b=0; b<4; ++b)
    {
        static const unsigned trans[4] = {0,2,1,3};
        unsigned transt = trans[b];
        
        char bit1 = 1&(palettebyte >> ((transt*2) +1));
        char bit2 = 1&(palettebyte >> ((transt*2) +0));
        
        c[b].palette = bit1*2 + bit2;
        c[b].tile    = ROM[offs + b];
    }
}

void Room::Write(unsigned ptroffs, unsigned offs) const
{
    ROM[ptroffs+0] = ((offs+0x8000) & 255);
    ROM[ptroffs+1] = ((offs+0x8000) >> 8);
    for(unsigned a=0; a<64; ++a) ROM[offs+a] = blocks[a];
}

void Room::Load(unsigned ptroffs)
{
    unsigned offs = ROM[ptroffs+0] | (ROM[ptroffs+1] << 8);
    offs = (ptroffs & 0x1C000) | (offs & 0x3FFF);
    for(unsigned a=0; a<64; ++a) blocks[a] = ROM[offs+a];
}
