static void CutmanAscii(Screen& s)
{
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
        {
            unsigned char c = s.outmap[y*16+x].tile;
            char ch = '?';
            /* Classify types.
             *
             *
             *
             */
            switch(c)
            {
                // air:
                case 0x00:
                case 0x01:
                    ch = '.';
                    break;
                case 0x23:
                case 0x24:
                case 0x25:
                case 0x26:
                case 0x2B:
                case 0x2C:
                case 0x34:
                case 0x35:
                case 0x3C:
                    ch = ' ';
                    break;

                case 0x62:
                case 0x68:
                case 0x69: // all kinds of walls
                case 0x51: case 0x52: case 0x53: // wily signature
                    ch = '#';
                    break;
                case 0x38:
                case 0x39:
                case 0x3A:
                case 0x3B:
                    ch = Ascii_Guts;
                    break;
                case 0x82:
                    ch = Ascii_Ladder;
                    break;
                case 0xC4:
                    ch = Ascii_Spike;
                    break;

                case 0x10:
                    ch = Ascii_WilyDoor;
                    break;
            }
            
            s.ascii[y*32+x] = ch;
            s.ascii[y*32+x+16] = ' ';
            //printf("%02X",c);
        }
}

static void CutmanChar(Screen& s)
{
    unsigned randseed = 0;
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
            randseed += s.ascii[y*32+x];
    std::srand(randseed);
    
    
    char decorations[16*16];
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
        {
            char c;
            switch(std::rand() % 6)
            {
                case 0: c = 1; break;
                case 1: c = 2; break;
                default: c = 0; break;
            }
            decorations[y*16+x] = c;
        }


    // Convert to tiles
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
        {
            unsigned char c   = 0;
            unsigned char pal = 0;
            
            /* Cutman palettes:
             *   
             * 0 = solid
             * 1 = blacks
             * 2 = blues
             * 3 = blue,gray
             */
            
            switch(s.ascii[y*32+x])
            {
                case '.':
                    c   = 0x01; // background
                    pal = 0x01;
                    break;
                case ' ':
                    if(y > 0 &&
                        (s.ascii[(y-1)*32+x] != '.'
                      && s.ascii[(y-1)*32+x] != ' ')
                      )
                        c = 0x25; // shadow
                    else
                    {
                        switch(decorations[y*16+x])
                        {
                            case 1:
                                c = 0x2C; // inset decoration
                                break;
                            case 2:
                                c = 0x35;
                                if(x<16
                                && decorations[y*16+x+1]==2
                                && (x==0 || decorations[y*16+x-1]!=2))
                                    c = 0x34; // beginning

                                if(x>0  && s.ascii[y*32+x-1] == ' '
                                && x<15 && s.ascii[y*32+x+1] == ' ')
                                {
                                    break;
                                }
                                // passthru
                            case 0:
                                c = 0x26; // no decoration
                        }
                    }
                    pal = 0x02;
                    break;
                case '#':
                    if(y > 0 &&
                        (s.ascii[(y-1)*32+x] != '#')
                      )
                        c = 0x62; // step
                    else
                        c = 0x68 + ((x+y) & 1);
                    pal = 0x00;
                    break;
                case Ascii_Ladder:
                    c   = 0x82; // ladder
                    pal = 0x00;
                    break;
                case Ascii_Spike:
                    c   = 0xC4; // spike
                    if(y>0 && s.ascii[(y-1)*32+x] == ' ')
                        pal = 0x03;
                    else
                        pal = 0x01;
                    break;
                case Ascii_Guts:
                    c   = 0x38 + (x&1) + (y&1)*2; // gutsblock
                    pal = 0x00;
                    break;
                case Ascii_WilyDoor:
                    c   = 0x10;
                    pal = 0x00;
                    break;
                default:
                    printf("? '%c'\n", s.ascii[y*32+x]);
            }
            
            s.outmap[y*16+x].tile    = c;
            s.outmap[y*16+x].palette = pal;
        }
}


static Level Cutman =
{
	0, // level number
	"Cutman",

	// screens:
	{
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"........GG......"   "                "
		"........GG......"   "                "
		"#####|####|#..##"   "                "
		"     |  ..|....."   "                "
		"     |  ..GG...."   "                "
		"     |  ..GG...."   "                "
		"     |  GGGG...."   "                "
		"     |  GGGG...."   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		,Down},

		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"........GG......"   " y              "
		"........GG......"   "     y          "
		"  ###|####|#..##"   "                "
		"     |  ..|....."   "                "
		"     |  ..GG...."   "    y           "
		"     |  ..GG...."   "                "
		"     |  GGGG...."   "                "
		"     |  GGGG...."   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		,Down},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "   y            "
		"................"   "        y       "
		"........GG......"   "                "
		". #.....GG......"   "                "
		"......####......"   "       y        "
		"......####..##.."   "                "
		"....######..##.."   "                "
		"....######..####"   "                "
		"  ########..####"   "                "
		"  ########..####"   "                "
		"  ########..####"   "                "
		"  ########..####"   "                "
		,Down},
		{
		"........    #|##"   "                "
		"........    #|##"   "                "
		"........     |##"   "                "
		"........     |##"   "                "
		"....#|#  #######"   "   <            "
		"....#|#  #######"   "                "
		"....#|      ####"   "                "
		". #.#|      ####"   "           <    "
		"....#|      ####"   "                "
		"....#|###   ####"   "                "
		"....       #####"   "                "
		"....       #####"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		,Down},
		{
		".............|##"   "                "
		".............|##"   "                "
		".............|##"   "                "
		"..........######"   "         {      "
		"..........######"   "                "
		"........########"   "       <        "
		"........########"   "                "
		". #...##########"   "                "
		"...#..##########"   "                "
		"...#....      ##"   "             <  "
		"........      ##"   "                "
		"........      ##"   "                "
		"  #.....      ##"   "                "
		"  ########  #|##"   "                "
		"  ########  #|##"   "                "
		"  ########  #|##"   "                "
		,Down},
		{
		"..........   |##"   "                "
		"..........   |##"   "                "
		"..........   |##"   "                "
		"..........  ####"   "           {    "
		"..........  ####"   "                "
		"...#......######"   "                "
		"...#......######"   "                "
		". #.....#|######"   "       <        "
		"........#|######"   "                "
		".........|    ##"   "                "
		".........|    ##"   "                "
		"...#.....|    ##"   "    >           "
		"  ##.....|    ##"   "                "
		"  ###...#####|##"   "                "
		"  ###...#####|##"   "                "
		"  ###...#####|##"   "                "
		,Down},
		{
		"...|  ##########"   "                "
		"...|   #########"   "                "
		"...|      ######"   "         <      "
		"...|       #####"   "                "
		"...|        ####"   "                "
		"...#####    ####"   "           {    "
		"....        ####"   "                "
		". #.        ####"   "                "
		"...#|###########"   "                "
		"....|       ####"   "                "
		"....|         ##"   "                "
		"....|         ##"   "                "
		"  #.|         ##"   "                "
		"  #####  ####|##"   "       >        "
		"  #####  ####|##"   "                "
		"  #####  ####|##"   "                "
		,Down},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		". #............."   "                "
		"......??????...."   "                "
		"......??????...."   "                "
		"......??????...."   "                "
		"......??????...."   "         r      "
		"  #...??????####"   "                "
		"  #|############"   "                "
		"  #|############"   "                "
		"  #|############"   "                "
		,Down},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"........GG......"   "                "
		". #.....GG......"   "       j        "
		"......####......"   "                "
		"......####..##.."   "     j          "
		"....######..##.."   "                "
		"....######..####"   "   j            "
		"  ########..####"   "                "
		"  ########..####"   "                "
		"  ########..####"   "                "
		"  ########..####"   "                "
		,Down},
		{
		"........    #|##"   "                "
		"........    #|##"   "                "
		"........     |##"   "                "
		"........     |##"   "                "
		"....#|#  #######"   "                "
		"....#|#  #######"   "                "
		"....#|      ####"   "      c         "
		". #.#|      ####"   "                "
		"....#|      ####"   "           h    "
		"....#|###   ####"   "                "
		"....       #####"   "    c           "
		"....       #####"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		,Down},
		{
		"..........   |##"   "                "
		"..........   |##"   "                "
		"..........   |##"   "                "
		"..........   |##"   "                "
		"...#......  ####"   "    h           "
		"..........    ##"   "                "
		"........##    ##"   "       h        "
		". ##|#..    ####"   "                "
		"....|...      ##"   "     c       h  "
		"....|...      ##"   "                "
		"....|.GG      ##"   "                "
		"....|.GG      ##"   "                "
		"  ###########|##"   "                "
		"  ###########|##"   "                "
		"  ###########|##"   "                "
		"  ###########|##"   "                "
		,Down},
		{
		"........      ##"   "                "
		"........      ##"   "                "
		"........      ##"   "                "
		"........      ##"   "                "
		"........      ##"   "             h  "
		"........      ##"   "                "
		"........    ####"   "           h    "
		". #.....    ####"   "                "
		"...G....  ######"   "                "
		"...G....  ######"   "                "
		"...###..########"   "       h        "
		"...###..      ##"   "        c       "
		"  ####..      ##"   "                "
		"  ###########|##"   "                "
		"  ###########|##"   "                "
		"  ###########|##"   "                "
		,Down},
		{
		"...|############"   "                "
		"...|############"   "                "
		"...|....      ##"   "            c   "
		"...|....      ##"   "                "
		"........      ##"   "  h             "
		"........      ##"   "                "
		"...#|...    ####"   "                "
		". #.|...    ####"   "   c            "
		"....|.GG    ####"   "        h       "
		"....|.GG    ####"   "                "
		"....############"   "   h            "
		"....############"   "                "
		"  #.############"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		,Down},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		". #............."   "                "
		"......??????...."   "                "
		"......??????...."   "                "
		"......??????...."   "                "
		"......??????...."   "         r      "
		"  #...??????..##"   "                "
		"  #|############"   "                "
		"  #|############"   "                "
		"  #|############"   "                "
		,Down},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"......##########"   "                "
		". #...##########"   "                "
		"...#############"   "                "
		"...#############"   "                "
		"...#############"   "                "
		"...#############"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		,Down},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "      y         "
		"................"   "          y     "
		"................"   "                "
		". #............."   "                "
		"...#########...."   "         y      "
		"...#########...."   "                "
		"................"   "                "
		"................"   "  E             "
		"  ##########|#.."   "                "
		"  ##########|#.."   "                "
		"  ##########|#.."   "                "
		"  ##########|#.."   "                "
		,Down},
		{
		"...###......|..."   "                "
		"...##.......|..."   "                "
		"...##.......|..."   "                "
		"...###|#######.."   "                "
		"...###|#######.."   "                "
		"...# ..........."   "                "
		"...# ..........."   "                "
		". #  ..........."   "                "
		"...  ..........."   "               u"
		"...  .##..##...."   "                "
		"...  ..........."   "                "
		"...  ..........."   "                "
		"  #  ..........."   "                "
		"  ####|#........"   "                "
		"  ####|#........"   "                "
		"  ####|#........"   "                "
		,Down},
		{
		"...###|#........"   "                "
		"...###|#........"   "                "
		"...###  ........"   "                "
		"...###  .....^^."   "                "
		"...#####.....##."   "                "
		"...#####........"   "                "
		"...#  .........."   "                "
		". ##  .........."   "                "
		"...   .........."   "               u"
		"...   .##....##."   "                "
		"...   .........."   "                "
		"...   .........."   "                "
		"  #   .........."   "                "
		"  ######^^^^^^^^"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		,Down},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		". #............."   "                "
		"...GGG.........."   "                "
		"...GGG.........."   "                "
		"...#GGGG........"   "              t "
		"...#GGGG........"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		,Down},

		{
		".............###"   "                "
		".............###"   "                "
		".............###"   "                "
		".............###"   "                "
		".............###"   "                "
		".............###"   "                "
		"..............DD"   "              SS"
		". #...........DD"   "              SS"
		"..............DD"   "              SS"
		"..............DD"   "              SS"
		"..........######"   "                "
		"..........######"   "                "
		"################"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		"  ##############"   "                "
		,Shutter},
		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"################"   "                "
		"################"   "                "
		"                "   "X               "
		"                "   "X               "
		"                "   "X               "
		"                "   "X               "
		"################"   "             v  "
		"################"   "                "
		".##............."   "                "
		"..#########....."   "                "
		"..##############"   "                "
		"................"   "                "
		,Right},
/*		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"################"   "                "
		"################"   "                "
		"                "   "            .   "
		"                "   "                "
		"                "   "                "
		"                "   "                "
		"################"   "                "
		"################"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		,Right},
*/		{
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"................"   "                "
		"################"   "                "
		"################"   "                "
		"               D"   "               S"
		"               D"   "               S"
		"                "   "                "
		"                "   "                "
		"################"   "     v          "
		"################"   "                "
		"................"   "                "
		"................"   "                "
		"################"   "                "
		"................"   "                "
		,Right},
		{
		"################"   "                "
		"################"   "                "
		"#              #"   "                "
		"#              #"   "                "
		"#              #"   "                "
		"#              #"   "                "
		"               #"   "X               "
		"               #"   "X               "
		"               #"   "                "
		"               #"   "                "
		"##GGGG         #"   "                "
		"##GGGG         #"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		"################"   "                "
		,End},
	},

	// palette
	{
		15, 48, 43,  6, 
		15, 48, 43, 44, 
		15, 32, 16,  0, 
		15, 32, 16, 44, 

		15, 44, 17, 15, 
		15, 32, 56, 15, 
		15, 48, 17, 15, 
		15, 48, 21, 15, 

		15, 48, 43,  6, 
		15, 48, 43, 44, 
		15, 48, 32, 16, 
		15, 32, 16, 44, 
	},

	// actives
	{
	},

	// enemies
	{
	},

	CutmanAscii, CutmanChar
};
