static void Wily1Ascii(Screen& s)
{
    GenericAsciiMaker(s,
// Boss room
-1,'X', 0x00,1,
// Suspense
-1,'V', 0x00,0, // entire black background
// Sky
-1,'.', 0x01,1,
// Decorations
-1,' ', 0x16,2, 0x17,2, // bg, shadow
-1,' ', 0x27,1, // grass
// Walls
-1,'#', 0x48,3, 0x49,3, 0x4A,3,
        0x50,3, 0x51,3, 0x52,3, // block type 1 (middle piece repeatable)
-1,'#', 0x4B,0, 0x4C,0, 0x4D,0,
        0x53,0, 0x54,0, 0x55,0, // block type 2 (middle piece repeatable)
-1,'#', 0x66,0, // floor outdoors
-1,'#', 0x4E,3, // floor indoors
// Spikes up, down (black bg)
-1,'^', 0xAE,0, 
-1,'^', 0xBF,0,
// Ladder
-1,'|', 0xC5,0,
// Gutsblock
-1,'G', 0x38,0, 0x39,0,
        0x3A,0, 0x3B,0,
-2);
}

static void Wily1Char(Screen& s)
{
    // Convert to tiles
    for(unsigned y=0; y<16; ++y)
        for(unsigned x=0; x<16; ++x)
        {
            unsigned char c = 0;
            unsigned char p = 0;
            
            /* Wily1 palettes:
             *   
             * 0 = solid
             * 1 = blacks
             * 2 = blues
             * 3 = blue,gray
             */
            
            switch(s.ascii[y*32+x])
            {
                case 'X': // boss room bg
                    c   = 0x00;
                    p   = 0x01;
                    break;
                case 'V': // black bg
                    c   = 0x00;
                    p   = 0x00;
                    break;
                case '.': // sky
                    c   = 0x01;
                    p   = 0x01;
                    break;
                case ' ':
                {
                    // If above is sky, this is grass.
                    if(y>0 && s.ascii[(y-1)*32 + x]=='.')
                    {
                        c = 0x27;
                        p = 0x01;
                        break;
                    }
                    c = 0x17;
                    p = 0x02;
                    // If above is solid, this is shadow.
                    if(y>0 && s.ascii[(y-1)*32 + x]=='#')
                    {
                        c = 0x16;
                    }
                    break;
                }
                case Ascii_Spike:
                {
                    // If below is solid, it's up. Otherwise it's down.
                    if(y<16 && s.ascii[(y+1)*32 + x]=='#')
                        c = 0xAE;
                    else
                        c = 0xBF;
                    p = 0x00;                    
                    break;
                }
                case Ascii_Ladder:
                {
                    c = 0xC5;
                    p = 0x00;
                    break;
                }
                case Ascii_Guts:
                {
                    c = 0x38 + (x&1) + (y&1)*2; // gutsblock
                    p = 0x00;
                    break;
                }
                case '#':
                {
                    // FIXME: do better
                    c = 0x4E;
                    p = 0x03;
                    break;
                }
                default:
                    printf("? '%c'\n", s.ascii[y*32+x]);
            }
            
            s.outmap[y*16+x].tile    = c;
            s.outmap[y*16+x].palette = p;
        }
}

static Level Wily1 =
{
    6, // level number
    "Wily1",

    // screens:
    {
        {
       //0123456789ABCDEF     0123456789ABCDEF 
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "                "   "                "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        ,Right},
        {
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "t               "
        "                "   "                "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        ,Right},
        {
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "               t"
        "                "   "                "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        ,Right},
        {
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "......####......"   "                "
        "......####......"   "                "
        "....########...."   "                "
        "    ########    "   "                "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        ,Right},
        {
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "                "
        "................"   "       t        "
        "................"   "                "
        "......####......"   "                "
        "......####......"   "                "
        "....########...."   "                "
        "    ########    "   "                "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        ,Right},
        {
        "..........######"   "                "
        "..........######"   "                "
        "..........######"   "                "
        "..........######"   "                "
        "..........######"   "                "
        "..........######"   "                "
        "..........######"   "                "
        "..........######"   "                "
        "..........GGGGGG"   "                "
        "..........GGGGGG"   "                "
        "..........GG##GG"   "                "
        "          GG##GG"   "                "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        ,Right},
        {
        "#############|##"   "                "
        "#############|##"   "                "
        "########### V|##"   "                "
        "########### V|##"   "                "
        "#########V  V###"   "                "
        "#########V  V###"   "                "
        "######V  V##X###"   "                "
        "######V  V##X###"   "                "
        "      V##V######"   "                "
        "      V##V######"   "                "
        "    ##X#########"   "            I   "
        "    ##X#########"   "                "
        "################"   "         I      "
        "################"   "                "
        "################"   "      I         "
        "################"   "                "
        ,Up},
        {
        "##|#############"   "                "
        "##|#############"   "                "
        "##|#####   #####"   "                "
        "##|#####   #####"   "         E      "
        "##|   ####    ##"   "                "
        "##|   ####    ##"   "                "
        "##|        ##|##"   "                "
        "##|        ##|##"   "                "
        "##           |##"   "                "
        "##   ##      |##"   "                "
        "##   ##      |##"   "                "
        "##   ##  #  #|##"   "                "
        "##^^^##^^#^^#|##"   "                "
        "#############|##"   "                "
        "#############|##"   "                "
        "#############|##"   "                "
        ,Up},
        {
        "################"   "                "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        "##              "   "                "
        "##              "   "                "
        "##              "   "                "
        "##              "   "                "
        "##            ##"   "       j   j   j"
        "##            ##"   "                "
        "##|#############"   "                "
        "##|#############"   "                "
        "##|#############"   "                "
        "##|#############"   "                "
        ,Right},
        {
        "################"   "                "
        "################"   "                "
        "        GGGG  ##"   "                "
        "        GGGG  ##"   "E  j   j        "
        "############  ##"   "                "
        "############  ##"   "                "
        "              ##"   "                "
        "              ##"   "             j  "
        "    ############"   "                "
        "    ############"   "                "
        "  GGGGGG      ##"   "                "
        "  GGGGGG      ##"   "                "
        "#############|##"   "                "
        "#############|##"   "                "
        "#############|##"   "                "
        "#############|##"   "                "
        ,Down},
        {
        "##            ##"   "                "
        "##            ##"   "                "
        "##            ##"   "                "
        "##  ############"   "                "
        "##            ##"   "                "
        "##            ##"   "                "
        "##            ##"   "                "
        "##            ##"   "                "
        "####  #  #  # ##"   "               s"
        "####^^#^^#^^# ##"   "                "
        "############# ##"   "                "
        "##            ##"   "                "
        "##            ##"   "                "
        "##|#############"   "                "
        "##|#############"   "                "
        "##|#############"   "                "
        ,Down},
        {
        "## #############"   "                "
        "## #############"   "                "
        "##  ^^^^^^^^^^^^"   "                "
        "##              "   "                "
        "##              "   "                "
        "##              "   "                "
        "##              "   "                "
        "##              "   "                "
        "##              "   "                "
        "##              "   "                "
        "##              "   "              T "
        "##              "   "                "
        "####            "   "                "
        "####^^^^^^^^^^^^"   "                "
        "################"   "                "
        "################"   "                "
        ,Right},
        {
        "######### ###|##"   "                "
        "######### ###|##"   "                "
        "^^^^^^^^^    |##"   "                "
        "             |##"   "                "
        "             |##"   "                "
        "             |##"   "                "
        "             |##"   "                "
        "             |##"   "                "
        "             |##"   "                "
        "             |##"   "                "
        "             |##"   "   L    L       "
        "             |##"   "           pp   "
        "          ######"   "                "
        "^^^^^^^^^^######"   "                "
        "################"   "                "
        "################"   "                "
        ,Up},
        {
        "##|#############"   "                "
        "##|#############"   "                "
        "##|   ##      ##"   "                "
        "##|   ##      ##"   "                "
        "##    ##  ##  ##"   "                "
        "##    ##  ##  ##"   "                "
        "##    ##  ##  ##"   "                "
        "##    ##  ##  ##"   "                "
        "##    ##  ##  ##"   "                "
        "##    ##  ##  ##"   "                "
        "##        ##  ##"   "                "
        "##        ##  ##"   "                "
        "##        ##  ##"   "                "
        "#########V###|##"   "                "
        "#########V###|##"   "                "
        "#########V###|##"   "                "
        ,Up},
        {
        "################"   "                "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        "##              "   "                "
        "##              "   "                "
        "##              "   "                "
        "##              "   "                "
        "##            ##"   "                "
        "##            ##"   "                "
        "##|#############"   "                "
        "##|#############"   "                "
        "##|#############"   "                "
        "##|#############"   "                "
        ,Shutter},
        {
        "################"   "                "
        "################"   "                "
        "XXXXXXXXXXXXXXX#"   "X               "
        "XXXXXXXXXXXXXXX#"   "X               "
        "XXXXXXXXXXXXXXX#"   "X               "
        "XXXXXXXXXXXXXXX#"   "X               "
        "XXXXXXXXXXXXXXX#"   "X               "
        "XXXXXXXXXXXXXXX#"   "X               "
        "XXXXXXXXXXXXXXX#"   "X               "
        "XXXXXXXXXXXXXXX#"   "X               "
        "################"   "X               "
        "################"   "X               "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        "################"   "                "
        ,End},
    },

    // palette
    {
        15, 32,  0,  2, 
        15, 43, 25, 44, 
        15, 23,  7, 15, 
        15, 32,  0, 38, 

        15, 44, 17, 15, 
        15, 32, 56, 15, 
        15, 39, 21, 15, 
        15, 48, 17, 15, 

        15, 32,  0,  2, 
        15,  0, 16, 32, 
        15, 23,  7,  7, 
        15, 32,  0, 38, 
    },

    // actives
    {
    },

    // enemies
    {
        Enemy( 6, 0xC8,0xA0, 0xF ),
        Enemy( 6, 0x98,0xC0, 0xF ),
        Enemy( 6, 0x68,0xE0, 0xF ),
        Enemy( 128, 0x00,0x00, 0x0 ),
    },

    Wily1Ascii, Wily1Char
};

